﻿namespace Vibstudio.NetConfigurator.Engine.Configuration
{
    #region Namespaces

    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Reflection;
    using System.Xml.Linq;

    using Vibstudio.NetConfigurator.Common;
    using Vibstudio.NetConfigurator.Extensions;
    using Vibstudio.NetConfigurator.Model;

    #endregion

    [Obfuscation(Exclude = true, ApplyToMembers = false)]
    internal class DbConfigurator : IConfiguration
    {
        #region Public Methods and Operators

        public bool Configure(string filePath, List<ConfigurationSetting> settings)
        {
            XDocument document = XDocument.Load(filePath);

            bool configured = ChangeConnectionString(document, settings);
            if (configured)
            {
                document.Save(filePath);
            }

            return configured;
        }

        public void Reset()
        {
            throw new NotImplementedException();
        }

        #endregion

        #region Methods

        private static bool ChangeConnectionString(XDocument document, IEnumerable<ConfigurationSetting> settings)
        {
            bool connectionStringChanged = true;

            try
            {
                IEnumerable<XElement> elements =
                    from e in document.Descendants("connectionStrings").Elements("add")
                    select e;

                foreach (ConfigurationSetting setting in settings)
                {
                    XElement element =
                        (from e in elements
                         where e.Attribute("name").Value == setting.Key
                         select e).FirstOrDefault();

                    if (element != null)
                    {
                        element.Attribute("connectionString").SetValue(setting.Value);
                    }
                }
            }
            catch
            {
                connectionStringChanged = false;
            }

            return connectionStringChanged;
        }

        #endregion
    }
}